
import { __decorate, __param } from '@codingame/monaco-vscode-api/external/tslib/tslib.es6';
import { Event } from '@codingame/monaco-vscode-api/vscode/vs/base/common/event';
import { VSBuffer } from '@codingame/monaco-vscode-api/vscode/vs/base/common/buffer';
import { deepClone } from '@codingame/monaco-vscode-api/vscode/vs/base/common/objects';
import { isPromptFile } from '@codingame/monaco-vscode-a1eb43d5-da63-5f76-8ad4-acf0d6c3b749-common/vscode/vs/platform/prompts/common/constants';
import { IStorageService } from '@codingame/monaco-vscode-api/vscode/vs/platform/storage/common/storage.service';
import { ITelemetryService } from '@codingame/monaco-vscode-api/vscode/vs/platform/telemetry/common/telemetry.service';
import { IUriIdentityService } from '@codingame/monaco-vscode-api/vscode/vs/platform/uriIdentity/common/uriIdentity.service';
import { IEnvironmentService } from '@codingame/monaco-vscode-api/vscode/vs/platform/environment/common/environment.service';
import { IConfigurationService } from '@codingame/monaco-vscode-api/vscode/vs/platform/configuration/common/configuration.service';
import { merge, areSame } from './promptsMerge.js';
import { AbstractSynchroniser } from '@codingame/monaco-vscode-a2719803-af40-5ae9-a29f-8a2231c33056-common/vscode/vs/platform/userDataSync/common/abstractSynchronizer';
import { FileOperationError, FileOperationResult } from '@codingame/monaco-vscode-api/vscode/vs/platform/files/common/files';
import { IFileService } from '@codingame/monaco-vscode-api/vscode/vs/platform/files/common/files.service';
import { SyncResource, USER_DATA_SYNC_SCHEME, Change } from '@codingame/monaco-vscode-api/vscode/vs/platform/userDataSync/common/userDataSync';
import { IUserDataSyncStoreService, IUserDataSyncLocalStoreService, IUserDataSyncLogService, IUserDataSyncEnablementService } from '@codingame/monaco-vscode-api/vscode/vs/platform/userDataSync/common/userDataSync.service';

function parsePrompts(syncData) {
    return JSON.parse(syncData.content);
}
let PromptsSynchronizer = class PromptsSynchronizer extends AbstractSynchroniser {
    constructor(profile, collection, environmentService, fileService, storageService, userDataSyncStoreService, userDataSyncLocalStoreService, logService, configurationService, userDataSyncEnablementService, telemetryService, uriIdentityService) {
        const syncResource = { syncResource: SyncResource.Prompts, profile };
        super(syncResource, collection, fileService, environmentService, storageService, userDataSyncStoreService, userDataSyncLocalStoreService, userDataSyncEnablementService, telemetryService, logService, configurationService, uriIdentityService);
        this.version = 1;
        this.promptsFolder = profile.promptsHome;
        this._register(this.fileService.watch(environmentService.userRoamingDataHome));
        this._register(this.fileService.watch(this.promptsFolder));
        this._register(Event.filter(this.fileService.onDidFilesChange, e => e.affects(this.promptsFolder))(() => this.triggerLocalChange()));
    }
    async generateSyncPreview(remoteUserData, lastSyncUserData, isRemoteDataFromCurrentMachine) {
        const local = await this.getPromptsFileContents();
        const localPrompts = this.toPromptContents(local);
        const remotePrompts = remoteUserData.syncData ? this.parsePrompts(remoteUserData.syncData) : null;
        lastSyncUserData = lastSyncUserData === null && isRemoteDataFromCurrentMachine ? remoteUserData : lastSyncUserData;
        const lastSyncPrompts = lastSyncUserData && lastSyncUserData.syncData ? this.parsePrompts(lastSyncUserData.syncData) : null;
        if (remotePrompts) {
            this.logService.trace(`${this.syncResourceLogLabel}: Merging remote prompts with local prompts...`);
        }
        else {
            this.logService.trace(`${this.syncResourceLogLabel}: Remote prompts does not exist. Synchronizing prompts for the first time.`);
        }
        const mergeResult = merge(localPrompts, remotePrompts, lastSyncPrompts);
        return this.getResourcePreviews(mergeResult, local, remotePrompts || {}, lastSyncPrompts || {});
    }
    async hasRemoteChanged(lastSyncUserData) {
        const lastSync = lastSyncUserData.syncData ? this.parsePrompts(lastSyncUserData.syncData) : null;
        if (lastSync === null) {
            return true;
        }
        const local = await this.getPromptsFileContents();
        const localPrompts = this.toPromptContents(local);
        const mergeResult = merge(localPrompts, lastSync, lastSync);
        return ( Object.keys(mergeResult.remote.added)).length > 0 || ( Object.keys(mergeResult.remote.updated)).length > 0 || mergeResult.remote.removed.length > 0 || mergeResult.conflicts.length > 0;
    }
    async getMergeResult(resourcePreview, token) {
        return resourcePreview.previewResult;
    }
    async getAcceptResult(resourcePreview, resource, content, token) {
        if (this.extUri.isEqualOrParent(resource, this.syncPreviewFolder.with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'local' }))) {
            return {
                content: resourcePreview.fileContent ? ( resourcePreview.fileContent.value.toString()) : null,
                localChange: Change.None,
                remoteChange: resourcePreview.fileContent
                    ? resourcePreview.remoteContent !== null ? Change.Modified : Change.Added
                    : Change.Deleted
            };
        }
        if (this.extUri.isEqualOrParent(resource, this.syncPreviewFolder.with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'remote' }))) {
            return {
                content: resourcePreview.remoteContent,
                localChange: resourcePreview.remoteContent !== null
                    ? resourcePreview.fileContent ? Change.Modified : Change.Added
                    : Change.Deleted,
                remoteChange: Change.None,
            };
        }
        if (this.extUri.isEqualOrParent(resource, this.syncPreviewFolder)) {
            if (content === undefined) {
                return {
                    content: resourcePreview.previewResult.content,
                    localChange: resourcePreview.previewResult.localChange,
                    remoteChange: resourcePreview.previewResult.remoteChange,
                };
            }
            else {
                return {
                    content,
                    localChange: content === null
                        ? resourcePreview.fileContent !== null ? Change.Deleted : Change.None
                        : Change.Modified,
                    remoteChange: content === null
                        ? resourcePreview.remoteContent !== null ? Change.Deleted : Change.None
                        : Change.Modified
                };
            }
        }
        throw ( new Error(`Invalid Resource: ${( resource.toString())}`));
    }
    async applyResult(remoteUserData, lastSyncUserData, resourcePreviews, force) {
        const accptedResourcePreviews = ( resourcePreviews.map(
            ([resourcePreview, acceptResult]) => ({ ...resourcePreview, acceptResult })
        ));
        if (accptedResourcePreviews.every(({ localChange, remoteChange }) => localChange === Change.None && remoteChange === Change.None)) {
            this.logService.info(`${this.syncResourceLogLabel}: No changes found during synchronizing prompts.`);
        }
        if (( accptedResourcePreviews.some(({ localChange }) => localChange !== Change.None))) {
            await this.updateLocalBackup(accptedResourcePreviews);
            await this.updateLocalPrompts(accptedResourcePreviews, force);
        }
        if (( accptedResourcePreviews.some(({ remoteChange }) => remoteChange !== Change.None))) {
            remoteUserData = await this.updateRemotePrompts(accptedResourcePreviews, remoteUserData, force);
        }
        if (lastSyncUserData?.ref !== remoteUserData.ref) {
            this.logService.trace(`${this.syncResourceLogLabel}: Updating last synchronized prompts...`);
            await this.updateLastSyncUserData(remoteUserData);
            this.logService.info(`${this.syncResourceLogLabel}: Updated last synchronized prompts`);
        }
        for (const { previewResource } of accptedResourcePreviews) {
            try {
                await this.fileService.del(previewResource);
            }
            catch (e) {  }
        }
    }
    getResourcePreviews(mergeResult, localFileContent, remote, base) {
        const resourcePreviews = ( new Map());
        for (const key of ( Object.keys(mergeResult.local.added))) {
            const previewResult = {
                content: mergeResult.local.added[key],
                hasConflicts: false,
                localChange: Change.Added,
                remoteChange: Change.None,
            };
            resourcePreviews.set(key, {
                baseResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'base' }),
                baseContent: null,
                fileContent: null,
                localResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'local' }),
                localContent: null,
                remoteResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'remote' }),
                remoteContent: remote[key],
                previewResource: this.extUri.joinPath(this.syncPreviewFolder, key),
                previewResult,
                localChange: previewResult.localChange,
                remoteChange: previewResult.remoteChange,
                acceptedResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'accepted' })
            });
        }
        for (const key of ( Object.keys(mergeResult.local.updated))) {
            const previewResult = {
                content: mergeResult.local.updated[key],
                hasConflicts: false,
                localChange: Change.Modified,
                remoteChange: Change.None,
            };
            const localContent = localFileContent[key] ? ( localFileContent[key].value.toString()) : null;
            resourcePreviews.set(key, {
                baseResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'base' }),
                baseContent: base[key] ?? null,
                localResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'local' }),
                fileContent: localFileContent[key],
                localContent,
                remoteResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'remote' }),
                remoteContent: remote[key],
                previewResource: this.extUri.joinPath(this.syncPreviewFolder, key),
                previewResult,
                localChange: previewResult.localChange,
                remoteChange: previewResult.remoteChange,
                acceptedResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'accepted' })
            });
        }
        for (const key of mergeResult.local.removed) {
            const previewResult = {
                content: null,
                hasConflicts: false,
                localChange: Change.Deleted,
                remoteChange: Change.None,
            };
            const localContent = localFileContent[key] ? ( localFileContent[key].value.toString()) : null;
            resourcePreviews.set(key, {
                baseResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'base' }),
                baseContent: base[key] ?? null,
                localResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'local' }),
                fileContent: localFileContent[key],
                localContent,
                remoteResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'remote' }),
                remoteContent: null,
                previewResource: this.extUri.joinPath(this.syncPreviewFolder, key),
                previewResult,
                localChange: previewResult.localChange,
                remoteChange: previewResult.remoteChange,
                acceptedResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'accepted' })
            });
        }
        for (const key of ( Object.keys(mergeResult.remote.added))) {
            const previewResult = {
                content: mergeResult.remote.added[key],
                hasConflicts: false,
                localChange: Change.None,
                remoteChange: Change.Added,
            };
            const localContent = localFileContent[key] ? ( localFileContent[key].value.toString()) : null;
            resourcePreviews.set(key, {
                baseResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'base' }),
                baseContent: base[key] ?? null,
                localResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'local' }),
                fileContent: localFileContent[key],
                localContent,
                remoteResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'remote' }),
                remoteContent: null,
                previewResource: this.extUri.joinPath(this.syncPreviewFolder, key),
                previewResult,
                localChange: previewResult.localChange,
                remoteChange: previewResult.remoteChange,
                acceptedResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'accepted' })
            });
        }
        for (const key of ( Object.keys(mergeResult.remote.updated))) {
            const previewResult = {
                content: mergeResult.remote.updated[key],
                hasConflicts: false,
                localChange: Change.None,
                remoteChange: Change.Modified,
            };
            const localContent = localFileContent[key] ? ( localFileContent[key].value.toString()) : null;
            resourcePreviews.set(key, {
                baseResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'base' }),
                baseContent: base[key] ?? null,
                localResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'local' }),
                fileContent: localFileContent[key],
                localContent,
                remoteResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'remote' }),
                remoteContent: remote[key],
                previewResource: this.extUri.joinPath(this.syncPreviewFolder, key),
                previewResult,
                localChange: previewResult.localChange,
                remoteChange: previewResult.remoteChange,
                acceptedResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'accepted' })
            });
        }
        for (const key of mergeResult.remote.removed) {
            const previewResult = {
                content: null,
                hasConflicts: false,
                localChange: Change.None,
                remoteChange: Change.Deleted,
            };
            resourcePreviews.set(key, {
                baseResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'base' }),
                baseContent: base[key] ?? null,
                localResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'local' }),
                fileContent: null,
                localContent: null,
                remoteResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'remote' }),
                remoteContent: remote[key],
                previewResource: this.extUri.joinPath(this.syncPreviewFolder, key),
                previewResult,
                localChange: previewResult.localChange,
                remoteChange: previewResult.remoteChange,
                acceptedResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'accepted' })
            });
        }
        for (const key of mergeResult.conflicts) {
            const previewResult = {
                content: base[key] ?? null,
                hasConflicts: true,
                localChange: localFileContent[key] ? Change.Modified : Change.Added,
                remoteChange: remote[key] ? Change.Modified : Change.Added
            };
            const localContent = localFileContent[key] ? ( localFileContent[key].value.toString()) : null;
            resourcePreviews.set(key, {
                baseResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'base' }),
                baseContent: base[key] ?? null,
                localResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'local' }),
                fileContent: localFileContent[key] || null,
                localContent,
                remoteResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'remote' }),
                remoteContent: remote[key] || null,
                previewResource: this.extUri.joinPath(this.syncPreviewFolder, key),
                previewResult,
                localChange: previewResult.localChange,
                remoteChange: previewResult.remoteChange,
                acceptedResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'accepted' })
            });
        }
        for (const key of ( Object.keys(localFileContent))) {
            if (!( resourcePreviews.has(key))) {
                const previewResult = {
                    content: localFileContent[key] ? ( localFileContent[key].value.toString()) : null,
                    hasConflicts: false,
                    localChange: Change.None,
                    remoteChange: Change.None
                };
                const localContent = localFileContent[key] ? ( localFileContent[key].value.toString()) : null;
                resourcePreviews.set(key, {
                    baseResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'base' }),
                    baseContent: base[key] ?? null,
                    localResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'local' }),
                    fileContent: localFileContent[key] || null,
                    localContent,
                    remoteResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'remote' }),
                    remoteContent: remote[key] || null,
                    previewResource: this.extUri.joinPath(this.syncPreviewFolder, key),
                    previewResult,
                    localChange: previewResult.localChange,
                    remoteChange: previewResult.remoteChange,
                    acceptedResource: this.extUri.joinPath(this.syncPreviewFolder, key).with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'accepted' })
                });
            }
        }
        return [...( resourcePreviews.values())];
    }
    async resolveContent(uri) {
        if (this.extUri.isEqualOrParent(uri, this.syncPreviewFolder.with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'remote' }))
            || this.extUri.isEqualOrParent(uri, this.syncPreviewFolder.with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'local' }))
            || this.extUri.isEqualOrParent(uri, this.syncPreviewFolder.with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'base' }))
            || this.extUri.isEqualOrParent(uri, this.syncPreviewFolder.with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'accepted' }))) {
            return this.resolvePreviewContent(uri);
        }
        return null;
    }
    async hasLocalData() {
        try {
            const local = await this.getPromptsFileContents();
            if (( Object.keys(local)).length) {
                return true;
            }
        }
        catch (error) {
        }
        return false;
    }
    async updateLocalBackup(resourcePreviews) {
        const local = {};
        for (const resourcePreview of resourcePreviews) {
            if (resourcePreview.fileContent) {
                local[this.extUri.basename(resourcePreview.localResource)] = resourcePreview.fileContent;
            }
        }
        await this.backupLocal(JSON.stringify(this.toPromptContents(local)));
    }
    async updateLocalPrompts(resourcePreviews, force) {
        for (const { fileContent, acceptResult, localResource, remoteResource, localChange } of resourcePreviews) {
            if (localChange !== Change.None) {
                const key = remoteResource ? this.extUri.basename(remoteResource) : this.extUri.basename(localResource);
                const resource = this.extUri.joinPath(this.promptsFolder, key);
                if (localChange === Change.Deleted) {
                    this.logService.trace(`${this.syncResourceLogLabel}: Deleting prompt...`, this.extUri.basename(resource));
                    await this.fileService.del(resource);
                    this.logService.info(`${this.syncResourceLogLabel}: Deleted prompt`, this.extUri.basename(resource));
                }
                else if (localChange === Change.Added) {
                    this.logService.trace(`${this.syncResourceLogLabel}: Creating prompt...`, this.extUri.basename(resource));
                    await this.fileService.createFile(resource, VSBuffer.fromString(acceptResult.content), { overwrite: force });
                    this.logService.info(`${this.syncResourceLogLabel}: Created prompt`, this.extUri.basename(resource));
                }
                else {
                    this.logService.trace(`${this.syncResourceLogLabel}: Updating prompt...`, this.extUri.basename(resource));
                    await this.fileService.writeFile(resource, VSBuffer.fromString(acceptResult.content), force ? undefined : fileContent);
                    this.logService.info(`${this.syncResourceLogLabel}: Updated prompt`, this.extUri.basename(resource));
                }
            }
        }
    }
    async updateRemotePrompts(resourcePreviews, remoteUserData, forcePush) {
        const currentPrompts = remoteUserData.syncData ? this.parsePrompts(remoteUserData.syncData) : {};
        const newPrompts = deepClone(currentPrompts);
        for (const { acceptResult, localResource, remoteResource, remoteChange } of resourcePreviews) {
            if (remoteChange !== Change.None) {
                const key = localResource ? this.extUri.basename(localResource) : this.extUri.basename(remoteResource);
                if (remoteChange === Change.Deleted) {
                    delete newPrompts[key];
                }
                else {
                    newPrompts[key] = acceptResult.content;
                }
            }
        }
        if (!areSame(currentPrompts, newPrompts)) {
            this.logService.trace(`${this.syncResourceLogLabel}: Updating remote prompts...`);
            remoteUserData = await this.updateRemoteUserData(JSON.stringify(newPrompts), forcePush ? null : remoteUserData.ref);
            this.logService.info(`${this.syncResourceLogLabel}: Updated remote prompts`);
        }
        return remoteUserData;
    }
    parsePrompts(syncData) {
        return parsePrompts(syncData);
    }
    toPromptContents(fileContents) {
        const prompts = {};
        for (const key of ( Object.keys(fileContents))) {
            prompts[key] = ( fileContents[key].value.toString());
        }
        return prompts;
    }
    async getPromptsFileContents() {
        const prompts = {};
        let stat;
        try {
            stat = await this.fileService.resolve(this.promptsFolder);
        }
        catch (e) {
            if (e instanceof FileOperationError && e.fileOperationResult === FileOperationResult.FILE_NOT_FOUND) {
                return prompts;
            }
            else {
                throw e;
            }
        }
        for (const entry of stat.children || []) {
            const resource = entry.resource;
            if (!isPromptFile(resource)) {
                continue;
            }
            const key = this.extUri.relativePath(this.promptsFolder, resource);
            const content = await this.fileService.readFile(resource);
            prompts[key] = content;
        }
        return prompts;
    }
};
PromptsSynchronizer = ( __decorate([
    ( __param(2, IEnvironmentService)),
    ( __param(3, IFileService)),
    ( __param(4, IStorageService)),
    ( __param(5, IUserDataSyncStoreService)),
    ( __param(6, IUserDataSyncLocalStoreService)),
    ( __param(7, IUserDataSyncLogService)),
    ( __param(8, IConfigurationService)),
    ( __param(9, IUserDataSyncEnablementService)),
    ( __param(10, ITelemetryService)),
    ( __param(11, IUriIdentityService))
], PromptsSynchronizer));

export { PromptsSynchronizer, parsePrompts };
