
import { __decorate, __param } from '@codingame/monaco-vscode-api/external/tslib/tslib.es6';
import { Extensions } from '@codingame/monaco-vscode-api/vscode/vs/workbench/common/contributions';
import { Registry } from '@codingame/monaco-vscode-api/vscode/vs/platform/registry/common/platform';
import { LifecyclePhase } from '@codingame/monaco-vscode-api/vscode/vs/workbench/services/lifecycle/common/lifecycle';
import { UserDataSyncWorkbenchContribution } from './userDataSync.js';
import { UserDataSyncErrorCode } from '@codingame/monaco-vscode-api/vscode/vs/platform/userDataSync/common/userDataSync';
import { IUserDataAutoSyncService } from '@codingame/monaco-vscode-api/vscode/vs/platform/userDataSync/common/userDataSync.service';
import '@codingame/monaco-vscode-api/vscode/vs/platform/notification/common/notification';
import { INotificationService } from '@codingame/monaco-vscode-api/vscode/vs/platform/notification/common/notification.service';
import { Disposable } from '@codingame/monaco-vscode-api/vscode/vs/base/common/lifecycle';
import { localize } from '@codingame/monaco-vscode-api/vscode/vs/nls';
import { isWeb } from '@codingame/monaco-vscode-api/vscode/vs/base/common/platform';
import { UserDataSyncTrigger } from './userDataSyncTrigger.js';
import { toAction } from '@codingame/monaco-vscode-api/vscode/vs/base/common/actions';
import { IProductService } from '@codingame/monaco-vscode-api/vscode/vs/platform/product/common/productService.service';
import { ICommandService } from '@codingame/monaco-vscode-api/vscode/vs/platform/commands/common/commands.service';
import { IHostService } from '@codingame/monaco-vscode-api/vscode/vs/workbench/services/host/browser/host.service';
import { SHOW_SYNC_LOG_COMMAND_ID } from '@codingame/monaco-vscode-api/vscode/vs/workbench/services/userDataSync/common/userDataSync';
import Severity from '@codingame/monaco-vscode-api/vscode/vs/base/common/severity';

let UserDataSyncReportIssueContribution = class UserDataSyncReportIssueContribution extends Disposable {
    constructor(userDataAutoSyncService, notificationService, productService, commandService, hostService) {
        super();
        this.notificationService = notificationService;
        this.productService = productService;
        this.commandService = commandService;
        this.hostService = hostService;
        this._register(userDataAutoSyncService.onError(error => this.onAutoSyncError(error)));
    }
    onAutoSyncError(error) {
        switch (error.code) {
            case UserDataSyncErrorCode.LocalTooManyRequests: {
                const message = isWeb ? ( localize(
                    11251,
                    "Settings sync is suspended temporarily because the current device is making too many requests. Please reload {0} to resume.",
                    this.productService.nameLong
                ))
                    : ( localize(
                    11252,
                    "Settings sync is suspended temporarily because the current device is making too many requests. Please restart {0} to resume.",
                    this.productService.nameLong
                ));
                this.notificationService.notify({
                    severity: Severity.Error,
                    message,
                    actions: {
                        primary: [
                            toAction({
                                id: 'Show Sync Logs',
                                label: ( localize(11253, "Show Log")),
                                run: () => this.commandService.executeCommand(SHOW_SYNC_LOG_COMMAND_ID)
                            }),
                            toAction({
                                id: 'Restart',
                                label: isWeb ? ( localize(11254, "Reload")) : ( localize(11255, "Restart")),
                                run: () => this.hostService.restart()
                            })
                        ]
                    }
                });
                return;
            }
            case UserDataSyncErrorCode.TooManyRequests: {
                const operationId = error.operationId ? ( localize(11256, "Operation Id: {0}", error.operationId)) : undefined;
                const message = ( localize(
                    11257,
                    "Settings sync is disabled because the current device is making too many requests. Please wait for 10 minutes and turn on sync."
                ));
                this.notificationService.notify({
                    severity: Severity.Error,
                    message: operationId ? `${message} ${operationId}` : message,
                    source: error.operationId ? ( localize(11258, "Settings Sync. Operation Id: {0}", error.operationId)) : undefined,
                    actions: {
                        primary: [
                            toAction({
                                id: 'Show Sync Logs',
                                label: ( localize(11253, "Show Log")),
                                run: () => this.commandService.executeCommand(SHOW_SYNC_LOG_COMMAND_ID)
                            })
                        ]
                    }
                });
                return;
            }
        }
    }
};
UserDataSyncReportIssueContribution = ( __decorate([
    ( __param(0, IUserDataAutoSyncService)),
    ( __param(1, INotificationService)),
    ( __param(2, IProductService)),
    ( __param(3, ICommandService)),
    ( __param(4, IHostService))
], UserDataSyncReportIssueContribution));
const workbenchRegistry = ( Registry.as(Extensions.Workbench));
workbenchRegistry.registerWorkbenchContribution(UserDataSyncWorkbenchContribution, LifecyclePhase.Restored);
workbenchRegistry.registerWorkbenchContribution(UserDataSyncTrigger, LifecyclePhase.Eventually);
workbenchRegistry.registerWorkbenchContribution(UserDataSyncReportIssueContribution, LifecyclePhase.Eventually);
